@@ -506,7 +506,7 @@ def state_present(module, existing, proposed, candidate):
     proposed_commands = apply_key_map(PARAM_TO_COMMAND_KEYMAP, proposed)
     existing_commands = apply_key_map(PARAM_TO_COMMAND_KEYMAP, existing)
 
-    for key, value in proposed_commands.iteritems():
+    for key, value in proposed_commands.items():
         if value is True:
             commands.append(key)
         elif value is False:
@@ -535,7 +535,7 @@ def state_absent(module, existing, proposed, candidate):
     parents = ['interface {0}'.format(module.params['interface'].capitalize())]
     existing_commands = apply_key_map(PARAM_TO_COMMAND_KEYMAP, existing)
 
-    for key, value in existing_commands.iteritems():
+    for key, value in existing_commands.items():
         if value:
             if key.startswith('ip ospf message-digest-key'):
                 if 'options' not in key:
@@ -606,6 +606,9 @@ def main():
                                                     'message_digest_password']],
                                 supports_check_mode=True)
 
+    if not module.params['interface'].startswith('loopback'):
+        module.params['interface'] = module.params['interface'].capitalize()
+
     for param in ['message_digest_encryption_type',
                   'message_digest_algorithm_type',
                   'message_digest_password']:
@@ -631,11 +634,11 @@ def main():
 
     existing = invoke('get_existing', module, args)
     end_state = existing
-    proposed_args = dict((k, v) for k, v in module.params.iteritems()
+    proposed_args = dict((k, v) for k, v in module.params.items()
                     if v is not None and k in args)
 
     proposed = {}
-    for key, value in proposed_args.iteritems():
+    for key, value in proposed_args.items():
         if key != 'interface':
             if str(value).lower() == 'true':
                 value = True
