@@ -25,7 +25,7 @@ module: set_fact
 short_description: Set host facts from a task
 description:
      - This module allows setting new variables.  Variables are set on a host-by-host basis just like facts discovered by the setup module.
-     - These variables will survive between plays during an Ansible run, but will not be saved across executions even if you use a fact cache.
+     - These variables will be available to subsequent plays during an ansible-playbook run, but will not be saved across executions even if you use a fact cache.
 options:
   key_value:
     description:
@@ -35,10 +35,13 @@ options:
     required: true
     default: null
 version_added: "1.2"
+notes:
+    - "The `var=value` notation can only create strings or booleans.
+      If you want to create lists/arrays or dictionary/hashes use `var: [val1, val2]`"
 '''
 
 EXAMPLES = '''
-# Example setting host facts using key=value pairs
+# Example setting host facts using key=value pairs, note that this always creates strings or booleans
 - set_fact: one_fact="something" other_fact="{{ local_var }}"
 
 # Example setting host facts using complex arguments
