@@ -108,7 +108,7 @@ EXAMPLES = '''
 
 import os
 import xml.etree.ElementTree as ET
-import urlparse
+import ansible.module_utils.six.moves.urllib.parse as urlparse
 
 from ansible.module_utils.basic import *
 from ansible.module_utils.ec2 import *
@@ -137,7 +137,7 @@ def create_tags_container(tags):
 
     tag_set = TagSet()
     tags_obj = Tags()
-    for key, val in tags.iteritems():
+    for key, val in tags.items():
         tag_set.add_tag(key, val)
 
     tags_obj.add_tag_set(tag_set)
