@@ -25,6 +25,7 @@ from ansible.compat.six import PY3
 from ansible.parsing.yaml.objects import AnsibleUnicode, AnsibleSequence, AnsibleMapping
 from ansible.parsing.yaml.objects import AnsibleVaultEncryptedUnicode
 from ansible.vars.hostvars import HostVars
+from ansible.vars.unsafe_proxy import AnsibleUnsafeText
 
 
 class AnsibleDumper(yaml.SafeDumper):
@@ -51,6 +52,11 @@ AnsibleDumper.add_representer(
     represent_unicode,
 )
 
+AnsibleDumper.add_representer(
+    AnsibleUnsafeText,
+    represent_unicode,
+)
+
 AnsibleDumper.add_representer(
     HostVars,
     represent_hostvars,
